-----------------------------------------------------------------------------------------------
-- Client Lua Script for Harvester
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "CraftingLib"
 
-----------------------------------------------------------------------------------------------
-- Harvester Module Definition
-----------------------------------------------------------------------------------------------

local Harvester = {}
 
local TradeskillLevels = {'Novice', 'Apprentice', 'Journeyman' ,'Artisan', 'Expert'}

local TradeskillNames = {}



local function GetCurrentLocale()
	local cancel = Apollo.GetString(1)
	-- French
	if cancel == "Annuler" then
		return "frFR"
	end
	-- German
	if cancel == "Abbrechen" then 
		return "deDE"
	end
	return "enUS"
end

local L    = _G["HarvesterLocales"][GetCurrentLocale()]
local L_EN = _G["HarvesterLocales"]["enUS"]

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Harvester:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function Harvester:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "Harvester"
	local tDependencies = {}

    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)

    self.filtersLoaded = false

    self.wasInCombat = false

    self.activeNodes = {}
    self.filteredNodes = {}
    self.ignoredNodes = {}

    self.typeFilters = {}
    self.activeFilters = {}

    self.settings = {
    	opacity = 1,
    	maxDistance = 300,
    	Options = {
	    	DisableAddon    = false,
    		HarvestableOnly = true,
    		HideInCombat    = false,
    		HideNoNodes     = false,
    		TrackMaster     = false,
    		UseMiniNav      = true
	    },
    	Farmer = {
    		spirovine     = true,
    		bladeleaf     = true,
    		pummelgranate = true,
    		yellowbell    = true,
    		serpentlily   = true,
			goldleaf      = true,
			honeywheat    = true,
			crowncorn     = true,
			coralscale    = true,
			logicleaf     = true,
			stoutroot     = true,
			glowmelon     = true,
			faerybloom    = true,
			witherwood    = true,
			flamefrond    = true,
			grimgourd     = true,
			bloodbriar    = true,
			mourningstar  = true,
			octopod       = true,
			heartichoke   = true
	    },
	    Tradeskills = {
	    	Mining = {
	    		Novice     = true,
	    		Apprentice = true,
	    		Journeyman = true,
	    		Artisan    = true,
	    		Expert     = true
		    },
	    	Survivalist = {
	    		Novice     = true,
	    		Apprentice = true,
	    		Journeyman = true,
	    		Artisan    = true,
	    		Expert     = true
		    },
	    	Relic_Hunter = {
	    		Novice     = true,
	    		Apprentice = true,
	    		Journeyman = true,
	    		Artisan    = true,
	    		Expert     = true
	    	}
		}
	}    
end
 

-----------------------------------------------------------------------------------------------
-- Harvester OnLoad
-----------------------------------------------------------------------------------------------
function Harvester:OnLoad()
	Apollo.RegisterEventHandler("UnitCreated", "OnUnitCreated", self)
	Apollo.RegisterEventHandler("UnitDestroyed", "OnUnitDestroyed", self)
	Apollo.RegisterEventHandler("UnitActivationTypeChanged", "OnUnitUpdated", self)
	Apollo.RegisterEventHandler("UnitEnteredCombat", "OnUnitCombat", self)

	self.xmlDoc = XmlDoc.CreateFromFile("Harvester.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- Harvester OnDocLoaded
-----------------------------------------------------------------------------------------------
function Harvester:OnDocLoaded()
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
	Apollo.RegisterEventHandler("OpenHarvesterConfiguration", "ToggleConfigurationWindow", self)

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		for code, id in pairs(CraftingLib.CodeEnumTradeskill) do
			TradeskillNames[code] = CraftingLib.GetTradeskillInfo(id).strName
		end

		Apollo.RegisterSlashCommand("harvester", "OnConfigure", self)
		Apollo.RegisterTimerHandler("OneSecTimer", "Update", self)
		self.arrowTimer = ApolloTimer.Create(4.000, true, "ShowArrow", self)

		self:OpenNavWindow()
	end
end

-----------------------------------------------------------------------------------------------
-- Harvester Functions
-----------------------------------------------------------------------------------------------

function Harvester:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Harvester", {"OpenHarvesterConfiguration", "", "IconSprites:Icon_Windows32_UI_CRB_InterfaceMenu_Tradeskills"})
end

function Harvester:ToggleConfigurationWindow()
	if self.wndOptions ~= nil and self.wndOptions:IsVisible() then
		self:CloseOptionsWindow()
	else
		self:OpenOptionsWindow()
	end
end

function Harvester:OnConfigure()
	self:OpenOptionsWindow()
end

function Harvester:InitTrackMaster()
	self.TM = Apollo.GetAddon("TrackMaster")
	if self.TM ~= nil then
		if self.settings.trackMasterLine == nil then
			self.settings.trackMasterLine = 1
		end
		self.trackMasterConfig = self.TM:AddToConfigMenu(self.TM.Type.Track, "Harvester", {
			CanFire = false,
			CanEnable = true,
			IsChecked = self.settings.Options.TrackMaster,
			OnEnableChanged = ( function(isEnabled)
				self.settings.Options.TrackMaster = isEnabled
				if not isEnabled and self.settings.trackMasterLine ~= nil then
					self.TM:SetTarget(nil, -1, self.settings.trackMasterLine)
				end
				if self.wndOptions ~= nil then
					self.wndOptions:FindChild("TrackMaster"):SetCheck(isEnabled)
				end
				self:ShowArrow()
			end ),
			LineNo = self.settings.trackMasterLine,
			OnLineChanged = ( function(lineNo)
				if self.settings.trackMasterLine ~= nil then
					self.TM:SetTarget(nil, -1, self.settings.trackMasterLine)
				end
				self.settings.trackMasterLine = lineNo
				self:ShowArrow()		
			end )
		})		
	end
end

function Harvester:InitNavWindows()
	self.navWindows = {
		NavWindow = Apollo.LoadForm(self.xmlDoc, "NavWindow", nil, self),
		MiniNavWindow = Apollo.LoadForm(self.xmlDoc, "MiniNavWindow", nil, self)
	}
    if self.settings.wndNavLocation ~= nil then
		self.navWindows.NavWindow:MoveToLocation(WindowLocation.new(self.settings.wndNavLocation))
	end
    if self.settings.wndMiniNavLocation ~= nil then
		self.navWindows.MiniNavWindow:MoveToLocation(WindowLocation.new(self.settings.wndMiniNavLocation))
	end

	self.navWindows.NavWindow:Show(false)
	self.navWindows.MiniNavWindow:Show(false)

	self.navWindows.NavWindow:SetSizingMinimum(185, 120)

	self.navWindows.MiniNavWindow:SetSizingMinimum(270, 48)
    self.navWindows.MiniNavWindow:SetSizingMaximum(504, 65)
end

function Harvester:OpenNavWindow()
	local windowName = "NavWindow"
	if self.settings.Options.UseMiniNav then
		windowName = "MiniNavWindow"
	end

	if self.navWindows == nil then
		self:InitNavWindows()
	end

	if self.wndNav ~= nil and self.wndNav:GetName() ~= windowName then
		self.wndNav:Show(false)
	end
	self.wndNav = self.navWindows[windowName]

    self.wndNav:Show(not self.settings.Options.DisableAddon)

    self.wndNav:SetOpacity(self.settings.opacity)

    if self.navActive then
	    self:StartNavigation()
    else
	    self:StopNavigation()
	end

    self:Update()
end

function Harvester:CloseNavWindow()
	self.settings.Options.DisableAddon = true
	self:LoadSettingsToForm()
	self:StopNavigation()
	self.wndNav:Close()
end

function Harvester:LoadOptionsLocale()
	local wndFarming = self.wndOptions:FindChild("Farmer")
	for key, value in pairs(L.farmingNodes) do
		wndFarming:FindChild(key):SetText(value)
	end

	local wndSettings = self.wndOptions:FindChild("Settings")
	for key, value in pairs(L.uiLines) do
		wndSettings:FindChild(key):SetText(value)
	end

	local info = self.wndOptions:FindChild("MaxDistance"):GetPixieInfo(1)
	info.strText = ""--L.uiLines.MaxDistance
	self.wndOptions:FindChild("MaxDistance"):UpdatePixie(1, info)

end

function Harvester:LoadSettingsToForm()
	if self.wndOptions == nil then
		return
	end

	local wndTS, wndOption 

	wndTS = self.wndOptions:FindChild("Settings")
	for key, val in pairs(self.settings.Options) do
		wndOption = wndTS:FindChild(key)
		if wndOption ~= nil then
			wndOption:SetCheck(val)
		end
	end

	for tradeskillName, levels in pairs(self.settings.Tradeskills) do
		wndTS = self.wndOptions:FindChild(tradeskillName)
		local isAnyChecked = false
		if wndTS ~= nil then
			for levelName, enabled in pairs(levels) do
				wndOption = wndTS:FindChild(levelName)
				if wndOption ~= nil then
					wndOption:SetCheck(enabled)
					if enabled then
						isAnyChecked = true
					end
				end
			end
			wndTS:FindChild("all"):SetCheck(isAnyChecked)
		end
	end

	wndTS = self.wndOptions:FindChild("Farmer")
	local isAnyChecked = false
	for name, enabled in pairs(self.settings.Farmer) do
		wndOption = wndTS:FindChild(name)
		if wndOption ~= nil then
			wndOption:SetCheck(enabled)
			if enabled then
				isAnyChecked = true
			end
		end
	end
	wndTS:FindChild("all"):SetCheck(isAnyChecked)

	self.wndOptions:FindChild("MaxDistance"):FindChild("SliderBar"):SetValue(self.settings.maxDistance)
	self:MaxDistanceChanged(nil, nil, self.settings.maxDistance, nil)	

	self.wndOptions:FindChild("Opacity"):FindChild("SliderBar"):SetValue(self.settings.opacity * 100)
	self:OpacityChanged(nil, nil, self.settings.opacity * 100, nil)	
end

function Harvester:OpenOptionsWindow()
	if self.wndOptions == nil then
		self.wndOptions = Apollo.LoadForm(self.xmlDoc, "OptionsWindow", nil, self)
		if self.wndOptions == nil then
			Apollo.AddAddonErrorText(self, "Could not load the options window for some reason.")
			return
		end
	    self.wndOptions:Show(false)
	    if self.settings.wndOptionsLocation ~= nil then
			self.wndOptions:MoveToLocation(WindowLocation.new(self.settings.wndOptionsLocation))
		end

		self.wndOptionsContent = Apollo.LoadForm(self.xmlDoc, "OptionsContent", self.wndOptions:FindChild("OptionsContent"), self)
		self:LoadOptionsLocale()
	end

	self:LoadSettingsToForm()

    self.wndOptions:Show(true)	
    self.wndOptionsContent:Show(true)
end

function Harvester:CloseOptionsWindow()
	self.wndOptions:Show(false)
end

function Harvester:GetItemIndex(list, item)
	for i, v in ipairs(list) do
		if v == item then
			return i
		end
	end
	return nil
end

function Harvester:TableMerge(t1, t2)
    for k,v in pairs(t2) do
    	if type(v) == "table" then
    		if type(t1[k] or false) == "table" then
    			self:TableMerge(t1[k] or {}, t2[k] or {})
    		else
    			t1[k] = v
    		end
    	else
    		t1[k] = v
    	end
    end
    return t1
end

function Harvester:RemoveItemFromList(list, item)
	local index = self:GetItemIndex(list, item)
	if index ~= nil then
		table.remove(list, index)
		return true
	end		
	return false
end

function Harvester:DistanceToNode(node)
	local player = GameLib.GetPlayerUnit()
	
	if (player == nil) then 
		return 0 
	end 
 
	local posPlayer = player:GetPosition()
	local posTarget = node:GetPosition()

	local nDeltaX = posTarget.x - posPlayer.x
	local nDeltaY = posTarget.y - posPlayer.y
	local nDeltaZ = posTarget.z - posPlayer.z
	
	return math.sqrt(math.pow(nDeltaX, 2) + math.pow(nDeltaY, 2) + math.pow(nDeltaZ, 2))
end

function Harvester:ClosestUnit()
	local closestUnit = nil
	local closestDistance = 1000
	for i, node in ipairs(self.filteredNodes) do
		if node ~= nil then
			local distance = self:DistanceToNode(node)
			if distance < closestDistance then
				closestUnit = node
				closestDistance = distance
			end
		end
	end
	return closestUnit
end

function Harvester:Filter(unit)
	local player = GameLib.GetPlayerUnit()
	if player == nil or not player:IsValid() or unit:GetType() ~= "Harvest"
	  or (self.settings.Options.HarvestableOnly and not unit:CanBeHarvestedBy(player))
	  or unit:IsDead()
	  or self:GetItemIndex(self.ignoredNodes, unit) ~= nil
	  or unit:GetName():find("Active ") ~= nil
	  or self:DistanceToNode(unit) > self.settings.maxDistance
	  then
		return false 
	end
	for i, filter in ipairs(self.activeFilters) do
		if unit:GetHarvestRequiredTradeskillName() == TradeskillNames[filter.tradeskillName] then
			if filter.nodeName ~= nil then
				if string.find(unit:GetName(), L.farmingNodes[filter.nodeName]) ~= nil or string.find(unit:GetName(), L_EN.farmingNodes[filter.nodeName]) ~= nil then
					return true
				end
			end

			if filter.tradeskillLevel ~= nil then
				if unit:GetHarvestRequiredTradeskillTier() == filter.tradeskillLevel then
					return true
				end
			end
		end
	end
	return false
end

function Harvester:StartNavigation()
	self.wndNav:FindChild("StartNav"):Show(false)
	self.wndNav:FindChild("StopNav"):Show(true)
	self.wndNav:FindChild("IgnoreNode"):Show(true)
	self.navActive = true
	self:Update()
	self:ShowArrow()
end

function Harvester:StopNavigation()
	self.wndNav:FindChild("StopNav"):Show(false)
	self.wndNav:FindChild("IgnoreNode"):Show(false)
	self.wndNav:FindChild("StartNav"):Show(true)
	self.navActive = false
end

function Harvester:AddNode(unit)
	if self:GetItemIndex(self.activeNodes, unit) == nil then
		table.insert(self.activeNodes, unit)
		if self:Filter(unit) then
			self:Update()
		end
	end
end

function Harvester:RemoveNode(unit)
	self:RemoveItemFromList(self.activeNodes, unit)
	if self:RemoveItemFromList(self.filteredNodes, unit) then
		self:Update()
	end
end

function Harvester:RemoveDeadNodes()
	for i, node in ipairs(self.activeNodes) do
		if node:IsDead() then
			if self:RemoveItemFromList(self.filteredNodes, unit) then
				self:Update()
			end
			table.remove(self.activeNodes, i)
			self:RemoveDeadNodes()
			break
		end
	end
end

function Harvester:IgnoreNode(node)
	if self:GetItemIndex(self.ignoredNodes, node) == nil then
		table.insert(self.ignoredNodes, node)
		self:Update()
	end 
end

function Harvester:IgnoreClosestNode()
	if self.closestNode ~= nil then
		self:IgnoreNode(self.closestNode)
	end
end

function Harvester:OnSave(saveLevel)
    if saveLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
    end

    if self.navWindows.NavWindow ~= nil then
	    self.settings.wndNavLocation = self.navWindows.NavWindow:GetLocation():ToTable()
	end

    if self.navWindows.MiniNavWindow ~= nil then
	    self.settings.wndMiniNavLocation = self.navWindows.MiniNavWindow:GetLocation():ToTable()
	end

    if self.wndOptions ~= nil then
	    self.settings.wndOptionsLocation = self.wndOptions:GetLocation():ToTable()
    end

    return self.settings
end

function Harvester:OnRestore(saveLevel, data)
	if saveLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then 
		return 
	end	
	self.settings = self:TableMerge(self.settings, data)

	self:LoadFilters()
end

function Harvester:OnUnitCreated(unit)
	self:AddNode(unit)
end

function Harvester:OnUnitUpdated(unit)
	self:AddNode(unit)
end

function Harvester:OnUnitDestroyed(unit)
	self:RemoveNode(unit)
end

function Harvester:OnUnitCombat(unit)
	if unit == GameLib.GetPlayerUnit() and self.settings.Options.HideInCombat then
		self:UpdateNavVisibility()
	end
end

function Harvester:ShowArrow()
	local player = GameLib.GetPlayerUnit()
	if player == nil then
		return
	end
	if (self.settings.Options.HideInCombat and player:IsInCombat())
		or self.closestNode == nil
		then
		if self.settings.Options.TrackMaster and self.TM ~= nil then
			self.TM:SetTarget(nil, -1, self.settings.trackMasterLine)
		end
		return
	end

	if self.navActive then
		if self.settings.Options.TrackMaster and self.TM ~= nil then 
			self.TM:SetTarget(self.closestNode, -1, self.settings.trackMasterLine)
		else
			self.closestNode:ShowHintArrow()
		end
	end
end

function Harvester:UpdateNavVisibility()
	local player = GameLib.GetPlayerUnit()
	if player == nil then
		return
	end
	if (self.settings.Options.HideInCombat and player:IsInCombat())
		or (self.settings.Options.HideNoNodes and self.closestNode == nil)
		or (self.settings.Options.DisableAddon) then

		if self.wndNav:IsVisible() then
			self.wndNav:Show(false)
			self:ShowArrow()
		end
	else
		if not self.wndNav:IsVisible() then
			self.wndNav:Show(true)
			self:ShowArrow()
		end
	end
end

function Harvester:Update()
	if self.settings.Options.DisableAddon then
		return
	end


	if not self.filtersLoaded then
		self:LoadFilters()
	end

	if self.settings.Options.TrackMaster and self.TM == nil then
		self:InitTrackMaster()
	end

	if self.wndNav == nil or GameLib.GetPlayerUnit() == nil then
		return
	end

	self:UpdateNavVisibility()

	self:RemoveDeadNodes()
	self.filteredNodes = {}
	for i, node in ipairs(self.activeNodes) do
		if self:Filter(node) then
			table.insert(self.filteredNodes, node)
		end
	end

	local exClosest = self.closestNode
	self.closestNode = self:ClosestUnit()
	if self.closestNode ~= nil then
		self.wndNav:FindChild("CurrentTarget"):SetText(self.closestNode:GetName())
		if self.navActive and exClosest ~= self.closestNode then
			self:ShowArrow()
		end
	else
		self.wndNav:FindChild("CurrentTarget"):SetText("Nothing nearby.")
		self:ShowArrow()
	end
end

function Harvester:CompareTables(t1, t2)
	for k, v in pairs(t1) do
		if t1[k] ~= t2[k] then
			return false
		end
	end
	for k, v in pairs(t2) do
		if t1[k] ~= t2[k] then
			return false
		end
	end
	return true
end

function Harvester:AddFilter(filter)
	table.insert(self.activeFilters, filter)
	self:Update()
end

function Harvester:RemoveFilter(filter)
	for index, activeFilter in ipairs(self.activeFilters) do
		if self:CompareTables(activeFilter, filter) then
			table.remove(self.activeFilters, index)
		end
	end
	self:Update()	
end

function Harvester:FarmingFilter(button)
	return { 
		tradeskillName = "Farmer", 
		nodeName = button:GetName()
	}
end

function Harvester:TradeskillFilter(button)
	local tradeskillName      = button:GetParent():GetName()
	local tradeskillLevelName = button:GetName()
	local tradeskillLevel     = self:GetItemIndex(TradeskillLevels, tradeskillLevelName)	

	return { 
		tradeskillName = tradeskillName,
		tradeskillLevel = tradeskillLevel
	}
end

function Harvester:CategoryToggle(wnd, button, mouseButton)
	local parent = button:GetParent()
	for i, checkbox in ipairs(parent:GetChildren()) do
		checkbox:SetCheck(button:IsChecked())
		if checkbox:GetName() ~= "all" then
			if parent:GetName() == "Farmer" then
				self:FarmingNodeToggle(nil, checkbox, nil)
			else
				self:TradeskillLevelToggle(nil, checkbox, nil)
			end
		end
	end
end

function Harvester:FarmingNodeToggle(wnd, button, mouseButton)
	if button:IsChecked() then
		self:AddFilter(self:FarmingFilter(button))
	else
		self:RemoveFilter(self:FarmingFilter(button))
	end
	self.settings.Farmer[button:GetName()] = button:IsChecked()

	if wnd ~= nil then
		local isAnyChecked = false
		for i, checkbox in ipairs(button:GetParent():GetChildren()) do
			if checkbox:GetName() ~= "all" and checkbox:IsChecked() then
				isAnyChecked = true
				break
			end
		end
		button:GetParent():FindChild("all"):SetCheck(isAnyChecked)	
	end
end

function Harvester:TradeskillLevelToggle(wnd, button, mouseButton)
	local tradeskillName      = button:GetParent():GetName()
	local tradeskillLevelName = button:GetName()
	local tradeskillLevel     = self:GetItemIndex(TradeskillLevels, tradeskillLevelName)

	self.settings.Tradeskills[tradeskillName][tradeskillLevelName] = button:IsChecked()
	if button:IsChecked() then
		self:AddFilter(self:TradeskillFilter(button))
	else
		self:RemoveFilter(self:TradeskillFilter(button))
	end

	if wnd ~= nil then
		local isAnyChecked = false
		for i, checkbox in ipairs(button:GetParent():GetChildren()) do
			if checkbox:GetName() ~= "all" and checkbox:IsChecked() then
				isAnyChecked = true
				break
			end
		end
		button:GetParent():FindChild("all"):SetCheck(isAnyChecked)
	end
end


function Harvester:TradeskillLevelChecked(wnd, button, mouseButton)
	local tradeskillName      = button:GetParent():GetName()
	local tradeskillLevelName = button:GetName()
	local tradeskillLevel     = self:GetItemIndex(TradeskillLevels, tradeskillLevelName)

	self.settings.Tradeskills[tradeskillName][tradeskillLevelName] = true

	self:AddFilter(self:TradeskillFilter(button))
end

function Harvester:TradeskillLevelUnchecked(wnd, button, mouseButton)
	local tradeskillName      = button:GetParent():GetName()
	local tradeskillLevelName = button:GetName()
	local tradeskillLevel     = self:GetItemIndex(TradeskillLevels, tradeskillLevelName)

	self.settings.Tradeskills[tradeskillName][tradeskillLevelName] = false

	self:RemoveFilter(self:TradeskillFilter(button))
end

function Harvester:OptionChecked(wnd, button, mouseButton)
	self.settings.Options[button:GetName()] = true

	if button:GetName() == 'TrackMaster' and self.trackMasterConfig ~= nil then
		self.trackMasterConfig:SetEnabled(true)
		self:ShowArrow()		
	end

	if button:GetName() == 'UseMiniNav' then
		self:OpenNavWindow()
	end

	if button:GetName() == "DisableAddon" then
		self:CloseNavWindow()		
	end

	self:Update()
end

function Harvester:OptionUnchecked(wnd, button, mouseButton)
	self.settings.Options[button:GetName()] = false

	if button:GetName() == 'TrackMaster' and self.trackMasterConfig ~= nil then
		self.trackMasterConfig:SetEnabled(false)
		if self.settings.trackMasterLine ~= nil then
			self.TM:SetTarget(nil, -1, self.settings.trackMasterLine)
		end
		self:ShowArrow()		
	end	

	if button:GetName() == 'UseMiniNav' then
		self:OpenNavWindow()
	end

	if button:GetName() == "DisableAddon" then
		self:OpenNavWindow()		
	end	

	self:Update()
end	

function Harvester:LoadFilters()
	local tradeskillLevel

	self.filtersLoaded = true
	for tradeskillName, levels in pairs(self.settings.Tradeskills) do
		for tradeskillLevelName, enabled in pairs(levels) do
			tradeskillLevel = self:GetItemIndex(TradeskillLevels, tradeskillLevelName)
			if enabled then
				local filter = { 
					tradeskillName = tradeskillName,
					tradeskillLevel = tradeskillLevel
				}
				self:AddFilter(filter)
			end
		end
	end

	for nodeId, enabled in pairs(self.settings.Farmer) do
		if enabled == true then
			local filter = { 
				tradeskillName = "Farmer", 
				nodeName = nodeId
			}
			self:AddFilter(filter)			
		end
	end
end

function Harvester:OpacityChanged( wndHandler, wndControl, fNewValue, fOldValue )
	self.wndOptions:FindChild("OpacityValue"):SetText(string.format("%.0f", fNewValue).."%")
	self.settings.opacity = fNewValue / 100
	if self.wndNav ~= nil then
		self.wndNav:SetOpacity(self.settings.opacity)
	end
end

function Harvester:MaxDistanceChanged( wndHandler, wndControl, fNewValue, fOldValue )
	self.wndOptions:FindChild("MaxDistanceValue"):SetText(string.format("%.0f", fNewValue).."m")
	self.settings.maxDistance = fNewValue
end

function Harvester:NavMouseEnter( wndHandler, wndControl, x, y )
	if self.wndNav ~= nil then
		self.wndNav:SetOpacity(1)
	end
end

function Harvester:NavMouseExit( wndHandler, wndControl, x, y )
	if self.wndNav ~= nil then
		self.wndNav:SetOpacity(self.settings.opacity)
	end
end

function Harvester:FilterCategoryChange( wndHandler, wndControl, eMouseButton )
	for i, btn in ipairs(self.wndOptions:FindChild(wndControl:GetName()):GetChildren()) do
		btn:SetCheck(wndControl:IsChecked())
		if wndControl:GetName() == "Farmer" then
			if wndControl:IsChecked() then
				self:FarmingNodeChecked(btn, btn, eMouseButton)
			else
				self:FarmingNodeUnchecked(btn, btn, eMouseButton)
			end

		end
	end
end

-----------------------------------------------------------------------------------------------
-- Harvester Instance
-----------------------------------------------------------------------------------------------




local HarvesterInst = Harvester:new()
HarvesterInst:Init()

